require 'spec_helper'
require 'moj/tribunals/config_loader'


describe Moj::Tribunals::ConfigLoader do
  describe '#initialize' do
    context 'with no arguments' do
      it 'defaults the config_path to default_config_path' do
        expect(subject.config_path).to eq(subject.send(:default_config_path))
      end
    end
    context 'given a config_path' do
      let(:config_path){ '/my/config_path' }
      subject{ Moj::Tribunals::ConfigLoader.new(config_path) }

      it 'stores the given config_path' do
        expect( subject.config_path ).to eq(config_path)
      end
    end
  end

  describe '#default_config_path' do
    let(:path_to_config){ File.join(File.dirname(__FILE__), '..', '..', '..', '..', 'config', 'tribunals') }
    let(:resolved_path_to_config){ File.expand_path( path_to_config ) }

    it 'is the full path to the config/tribunals folder' do
      expect(subject.send(:default_config_path)).to eq(resolved_path_to_config)
    end
  end

  describe '#load_file' do
    it 'loads the given file as YAML' do
      expect(YAML).to receive(:load_file).with('/my/file')
      subject.send(:load_file, '/my/file')
    end

    it 'returns the result of parsing the given file as YAML' do
      allow(YAML).to receive(:load_file).with('/my/file').and_return( {key: 'value'} )
      expect(subject.send(:load_file, '/my/file')).to eq({key: 'value'})
    end
  end

  describe '#config_files' do
    it 'returns all YAML files in the config folder' do
      expect(subject.config_files.sort.map{|f| File.basename(f) } ).to eq(['care-standards.yml', 'eat.yml', 'ftt-tax.yml', 'utiac.yml'])
    end
  end

  describe '#config_file_for' do
    context 'given a tribunal name that exists as a config file' do
      let(:tribunal){ 'ftt-tax' }

      it 'returns the full path to the corresponding config file' do
        expect(subject.config_file_for(tribunal)).to eq(File.join(subject.send(:config_path), "#{tribunal}.yml"))
      end
    end

    context 'given a tribunal name that exists as a config file' do
      let(:tribunal){ 'non_existent_tribunal' }

      it 'returns the full path to the corresponding config file' do
        expect(subject.config_file_for(tribunal)).to be_nil
      end
    end
  end

  describe '#load' do
    it 'returns a hash' do
      expect(subject.load).to be_a(Hash)
    end

    describe 'the returned Hash' do
      let(:result){ subject.load }

      describe 'each tribunal' do
        let(:tribunals){ [:'care-standards', :eat, :'ftt-tax', :utiac] }

        it 'has a top-level key' do
          expect(result.keys.map(&:to_sym)).to eq( tribunals )
        end

        it 'has second-level keys for each tribunal attribute in the YAML file' do
          tribunals.each do |tribunal|
            expect(result[tribunal].keys.map(&:to_sym)).to include( :name, :title, :search_params )
          end
        end
      end
    end
  end
end